require 'spec_helper'
require 'shortcodes'

describe Shortcodes::TutsShortcodes do
  subject(:shortcodes) { Shortcodes::TutsShortcodes.new }

  describe '[xml]' do
    it 'returns an appropriate pre tag for xml' do
      xml_shortcode = '
      [xml]
        <div class="class">
          Some Text
        </div>
      [/xml]
      '
      expected = '
      <pre class="brush: xml">
        <div class="class">
          Some Text
        </div></pre>
      '
      shortcodes.process(xml_shortcode).should == expected
    end
  end

  describe '[sponsored_content]' do
    it 'returns an appropriate div tag for sponsored content' do
      shortcode = '
      [sponsored_content]
        <a href="http://www.filterforge.com/?psdtuts" onclick="javascript:_gaq.push([\'_trackEvent\',\'outbound-article\',\'www.filterforge.com\']);" sl-processed="1">Filter Forge</a>
      [/sponsored_content]
      '
      expected = '<div class="sponsored-shortcode"><strong>Sponsored Content</strong><p>This content was commissioned by <a href="http://www.filterforge.com/?psdtuts" onclick="javascript:_gaq.push([\'_trackEvent\',\'outbound-article\',\'www.filterforge.com\']);" sl-processed="1">Filter Forge</a> and was written and/or edited by the Tuts+ team. Our aim with sponsored content is to publish relevant and objective tutorials, case studies, and inspirational interviews that offer genuine educational value to our readers and enable us to fund the creation of more useful content.</p></div>'

      shortcodes.process(shortcode).strip.should == expected
    end
  end

  describe '[sourcecode]' do
    it 'returns an appropriate pre tag for the given language attribute of sourcecode' do
      bash_shortcode = '
      [sourcecode language="php"]
        some php code
      [/sourcecode]
      '
      expected = '
      <pre class="brush: php">
        some php code</pre>
      '
      shortcodes.process(bash_shortcode).should == expected
    end

    it 'returns an appropriate pre tag for the given lang attribute of sourcecode' do
      bash_shortcode = '
      [sourcecode lang="php"]
        some php code
      [/sourcecode]
      '
      expected = '
      <pre class="brush: php">
        some php code</pre>
      '
      shortcodes.process(bash_shortcode).should == expected
    end

    it 'uses plain brush as its default syntax if language attribute is not given' do
      bash_shortcode = '
      [sourcecode]
        some random code
      [/sourcecode]
      '
      expected = '
      <pre class="brush: plain">
        some random code</pre>
      '
      shortcodes.process(bash_shortcode).should == expected
    end

    it 'returns nothing for a content-less shortcode tag' do
      shortcode = '[sourcecode language="php"]'
      expected  = ''
      shortcodes.process(shortcode).should == expected
    end
  end

  describe '[code]' do
    it 'behaves the same as sourcecode' do
      bash_shortcode = '
      [code language="php"]
        some php code
      [/code]
      '
      expected = '
      <pre class="brush: php">
        some php code</pre>
      '
      shortcodes.process(bash_shortcode).should == expected
    end
  end

  describe '[tip]' do
    let(:source) { '[tip]In doing something for the first time, imagine that you have already
done it in the past.[/tip]' }
    let(:expected) { '<div class="tip-shortcode">In doing something for the first time, imagine that you have already
done it in the past.</div>' }

    it 'wraps the tip in a div (WTF)' do
      shortcodes.process(source).should == expected
    end

    context 'when tip has no content' do
      let(:source) { '[tip]' }

      it 'returns nothing' do
        shortcodes.process(source).should == ''
      end
    end
  end

  describe '[polldaddy]' do
    let(:source) { '[polldaddy id="3382283" title="Whee whoo haha"]' }
    let(:expected) { '<script type="text/javascript" charset="utf-8" src="http://static.polldaddy.com/p/3382283.js"></script>'}

    it 'produces a polldaddy script tag' do
      shortcodes.process(source).should == expected
    end
  end

  describe '[audio]' do

    context "when type not specified" do
      let(:source) { '[audio http://hello.com]' }
      let(:expected) { '<audio src="http://hello.com"  />'}

      it 'produces an audio tag' do
        shortcodes.process(source).should == expected
      end
    end

    context "when audio is separated from the tag with a colon with no space" do
      let(:source) { '[audio:http://hello.com]' }
      let(:expected) { '<audio src="http://hello.com"  />'}

      it 'produces an audio tag' do
        shortcodes.process(source).should == expected
      end
    end

    context "when audio is separated from the tag with a colon and whitespace" do
      let(:source) { '[audio: http://hello.com]' }
      let(:expected) { '<audio src="http://hello.com"  />'}

      it 'produces an audio tag' do
        shortcodes.process(source).should == expected
      end

      context "and the URL contains whitespace" do
        let(:source) { '[audio: http://how silly.com]' }
        let(:expected) { '<audio src="http://how silly.com"  />'}

        it "retains the full path with the whitespace" do
          shortcodes.process(source).should == expected
        end
      end
    end

    context "when type is specified as mp3" do
      let(:source) { "[audio mp3='http://hello.com']" }
      let(:expected) { %Q[<audio src="http://hello.com" type='audio/mpeg; codecs=\"mp3\"' />]}

      it 'produces an audio tag w/ an mp3 codec' do
        shortcodes.process(source).should == expected
      end
    end

    context "when type is specified as ogg" do
      let(:source) { "[audio ogg='http://hello.com']" }
      let(:expected) { %Q[<audio src="http://hello.com" type='audio/ogg; codecs=\"vorbis\"' />]}

      it 'produces an audio tag w/ an ogg codec' do
        shortcodes.process(source).should == expected
      end
    end

  end

  describe '[WP_UnityObject]' do
    let(:source) { '[WP_UnityObject src="http://activetuts.s3.amazonaws.com/tuts/448_unity4/preview/WebPlayer.unity3d" width="600" height="450" /]' }
    let(:expected) { %{<script src="http://webplayer.unity3d.com/download_webplayer-3.x/3.0/uo/UnityObject2.js"></script><script async>$(function(){new UnityObject2().initPlugin($('.unity-player')[0], "http://activetuts.s3.amazonaws.com/tuts/448_unity4/preview/WebPlayer.unity3d");});</script><div class="unity-player" style="width:600px;height:450px"><div class="unity-player__missing"><a href="http://unity3d.com/webplayer/"><img alt="Unity Web Player. Install now!" src="http://webplayer.unity3d.com/installation/getunity.png" width="193" height="63" /></a></div></div>} }

    it 'produces a unity object tag' do
      shortcodes.process(source).should == expected
    end
  end

  describe '[caption]' do
    context 'with valid image and caption' do
      let(:image) { '<img src="http://localhost/wp-content/uploads/2010/07/800px-Great_Wave_off_Kanagawa2-300x205.jpg" alt="Kanagawa" title="The Great Wave" width="300" height="205" class="size-medium wp-image-6" />' }
      let(:caption) { 'WordPress shortcodes always exist for a good reason' }
      let(:source) { %([caption id="attachment_6" align="alignright" width="300"]#{image} #{caption}[/caption]) }

      it 'wraps content in figure and figcaption tags' do
        expect(shortcodes.process(source)).to eq %(<figure class="post_image">#{image}<figcaption>#{caption}</figcaption></figure>)
      end
    end

    context 'with invalid content' do
      let(:content) { 'All content is valid content' }
      let(:source) { %([caption id="attachment_5" align="alignright" width="300"]#{content}[/caption]) }

      it 'returns content' do
        expect(shortcodes.process(source)).to eq content
      end
    end
  end

  describe '[player]' do
    let(:source) { '[player href="http://s3.amazonaws.com/nettuts/631_myows/apiRegister.flv" width="600" height="494"]' }
    let(:expected) { %{<video class="post__content-video" src="http://s3.amazonaws.com/nettuts/631_myows/apiRegister.flv" width="600" height="494" />} }

    # This relies on some html5 player being initialised for all <video> tags. In the case of Hub at the time of writing
    # we already have mediaelement loaded for Courses videos, so this just works.
    it 'produces a video tag' do
      shortcodes.process(source).should == expected
    end
  end
end
